import Foundation

/**
Ermittelt günstige Paarungen von Coaches und Kandidaten für den Songwettbewerb

- author: Albert Wiedemann
- version: 1.0
*/
class Songwettbewerb
{
    /** Die Wunschlisten der Kandidaten für ihre Coaches */
    private var kandidatenWunschlisteCoaches: [[Int]]
    /** Die Wunschlisten der Coaches für ihre Kandidaten */
    private var coachesWunschlisteKandidaten: [[Int]]
    /** Das Feld der zugeordneten Coaches (-1 bedeutert, dass dem Kandidaten noch kein Coach zugeordnet wurde) */
    private var coachFürKandidat: [Int]
    /** Anzahl der Kandidaten und Coaches = maximaler Wunschwert */
    private var anzahl: Int

    /**
	Legt die Grunddaten an.
    */
    init()
    {
        kandidatenWunschlisteCoaches = [[Int]]()
        coachesWunschlisteKandidaten = [[Int]]()
        anzahl = coachesWunschlisteKandidaten.count
        coachFürKandidat = [Int](repeating: -1, count: anzahl)
        if !Lesen(name: "Wunschlisten.txt", kandidatenWunschlisteCoaches: &kandidatenWunschlisteCoaches, coachesWunschlisteKandidaten: &coachesWunschlisteKandidaten)
        {
            return
        }
        SchrittAusführen(kandidat: 0)
    }
    
    /**
	Liest die Wunschlistendaten aus der angegebenen Datei
	- parameters:
		- name: der Dateiname
		- kandidatenWunschlisteCoaches: Wunschlisten der Kandidaten für ihre Coaches
 		- coachesWunschlisteKandidaten: Wunschlisten der Coaches für ihre Kandidaten
	- returns: wahr, wenn das Lesen erfolgreich war
    */
    private func Lesen (name: String, kandidatenWunschlisteCoaches: inout [[Int]], coachesWunschlisteKandidaten: inout [[Int]]) -> Bool
    {
        do
        {
            guard let datei = Bundle.main.url(forResource: name, withExtension: "")
            else
            {
            	print ("Die Datei \(name) exisitert nicht")
            	return false
            }
            let zeilen = try String(contentsOf: datei, encoding: .utf8).split(separator: "\n")
			var zeilenNummer = 0
            if zeilen[zeilenNummer].first != "*"
            {
                print("Falsche Dateistruktur Start")
                return false
            }
            zeilenNummer += 1
            var pos = -1
            while zeilen[zeilenNummer].first != "*"
            {
                let teile = zeilen[zeilenNummer].split(separator: ",")
                kandidatenWunschlisteCoaches.append([Int]())
                pos += 1
                for t in teile
                {
                    var num = String(t)
                    num.removeAll(where: {$0 == " "})
                    if let wert = Int(num)
                    {
                        kandidatenWunschlisteCoaches[pos].append(wert)
                    }
                    else
                    {
                    	print("Falsche Dateistruktur Lesen 1")
                    	return false
                    }
                }
                zeilenNummer += 1
            }
            zeilenNummer += 1
            pos = -1
            while zeilen[zeilenNummer].first != "*"
            {
                let teile = zeilen[zeilenNummer].split(separator: ",")
                coachesWunschlisteKandidaten.append([Int]())
                pos += 1
                for t in teile
                {
                    var num = String(t)
                    num.removeAll(where: {$0 == " "})
                    if let wert = Int(num)
                    {
                        coachesWunschlisteKandidaten[pos].append(wert)
                    }
                    else
                    {
                    	print("Falsche Dateistruktur Lesen 2")
                    	return false
                    }
                }
                zeilenNummer += 1
            }
            return kandidatenWunschlisteCoaches.count == coachesWunschlisteKandidaten.count
        }
        catch
        {
            print("Fehler beim Lesen der Datei: \(error)")
            return false
        }
    }

    /**
	Gibt eine gefundene Lösung aus
    */
    private func LösungAusgeben()
    {
        for coach in coachFürKandidat
        {
            print("   \(coach)".suffix(3), terminator: "")
        }
        print()
        // Gemäß Teilaufgabe d ergänzen
    }
    
	/**
	Sucht für den angegebenen Kandidaten den Coach, den der Kandidat mit dem gegebenen Wunschwert haben möchte)
	- parameters:
		- kandidat: der Kandidat, für den der Coach gesucht wird
		- wunschwert: der Wunschwert für den Coach
	- returns: der Coach zum angegeben Wunschwert
	*/
    private func CoachFürWunschwertGeben(kandidat: Int, wunschwert: Int) -> Int
    {
        for nummer in 0 ..< anzahl
        {
            if kandidatenWunschlisteCoaches[kandidat][nummer] == wunschwert
            {
                return nummer
            }
        }
        return -1
    }
    
    /**
	Sucht denjenigen Kandidaten, dem der Coach zur Zeit zugeordnet ist.
	- parameters:
		- coach: Coach, dessen Kandidat gesucht wird
	- returns: der zugeordnete Kandidat
     */
    private func KandidatFürCoachGeben(coach: Int) -> Int
    {
        for kandidat in 0 ..< anzahl
        {
            if coachFürKandidat[kandidat] == coach
            {
                return kandidat
            }
        }
        return -1
    }
    
    /**
	Testet, ob die Aufstellung nach Hinzufügen des Paares Kandidat/Coach stabil bleibt
	- parameters:
		- aktuellerKandidat: der Kandidat
		- aktuellerCoach: und dessen geplanter Coach
	- returns: wahr, wenn die Aufstellung stabil bleibt
    */
    private func BleibtStabil(aktuellerKandidat: Int, aktuellerCoach: Int) -> Bool
    {
        for kandidat in 0	..< aktuellerKandidat
        {
            if (kandidatenWunschlisteCoaches[kandidat][aktuellerCoach] > kandidatenWunschlisteCoaches[kandidat][coachFürKandidat[kandidat]]) &&
                (coachesWunschlisteKandidaten[aktuellerCoach][kandidat] > coachesWunschlisteKandidaten[aktuellerCoach][aktuellerKandidat])
            {
                return false
            }
        }
        // Gemäß Teilaufgabe c ergänzen
        return true
    }

    /**
	Sucht einen Coach für den nächsten Kandidaten
	- parameters:
		- kandidat: der nächste Kandidat
     */
    private func SchrittAusführen(kandidat: Int)
    {
        // Gemäß Teilaufgabe b ergänzen
    }
}

Songwettbewerb()

